#ifndef __CYCLE_CYCLE
#define __CYCLE_CYCLE

// Cycle() adds new methods and members to a list to make it a cycle list
// with a "selected item" that keeps track of where you are in that list (the
// "current item").
function Cycle(list) {
  this.list = list || [];
  this.selected_index = 0;
};

Cycle.prototype = {
  get length() { return this.list.length; }
};

// Advance() cycles selected_index through the list by delta positions (or 1
// position if delta is not set).  Returns true if the current index changed.
Cycle.prototype.Advance = function(delta) {
  if (delta == undefined)
    delta = 1;

  var initial_index = this.selected_index;
  if (this.list.length && delta) {
    this.selected_index = (this.selected_index + delta) % this.list.length;

    if (this.selected_index < 0)
      this.selected_index += this.list.length;
  }
  return initial_index != this.selected_index;
};


  // Recursively get the current item from nested cycle lists.
Cycle.prototype.Get = function(depth) {
  if (depth == undefined)
    depth = 1;

  if (depth == 0)
    return this;

  var item = this.list[this.selected_index];
  return (depth == 1) ? item : item.Get(depth - 1);
};

// Cycle.Depth computes the depth of a Cycle - the number of Get()s before we
// come to something that's not a Cycle.
Cycle.Depth = function(c) {
  if (!(typeof(cycle) == 'object' && cycle.prototype === Cycle.prototype))
    return 0;

  if (!c.length)
    return 1;

  return 1 + Cycle.Depth(c.Get());
};

Cycle.prototype.Depth = function() {
  return Cycle.Depth(this);
};

Cycle.prototype.Insert = function(item, index) {
  if (index === undefined)
    index = this.selected_index;

  var ls = this.list;
  this.list = ls.slice(0, index + 1).concat([item]).concat(ls.slice(index + 1));

  if (this.length > 1 && this.selected_index >= index)
    ++this.selected_index;
};

Cycle.prototype.Remove = function(index) {
  if (index === undefined)
    index = this.selected_index;

  this.list = this.list.slice(0, index).concat(this.list.slice(index + 1));
  if (this.selected_index && this.selected_index > index)
    --this.selected_index;
};

Cycle.Encycle = function(depth, item) {
  item = item || [];
  for (var i = 0; i < depth; ++i)
    item = new Cycle([item]);

  return item;
};

#endif  // __CYCLE_CYCLE

