#ifndef __FUNCTIONAL
#define __FUNCTIONAL

#include "util/array.js"

var Functional = new function() {
  // I define a "Functional" as either a function or a map (which is any
  // Javascript object).

  // Apply returns f(x) or f[x], appropriately.
  this.Apply = function(f, x) {
    return f && ((typeof(f) == 'function') ? f(x) : f[x]);
  };

  // Chain calls Apply(f, x) for each value given, chaining results so that the
  // result of the previous step is the functional used in the next step.
  this.Chain = function(f, values) {
    if (!Array.Is(values))
      values = [values];

    for (var i = 0; i < values.length; ++i)
      f = f && this.Apply(f, values[i]);

    return f;
  };

  // Partial(f) partially binds arguments to the front or back of f, so that,
  // for example:
  //
  //   Partial(f)(a, b, c)(d, e, f) := f(a, b, c, d, e, f).
  this.Partial = function(f) {
    return function(_) {
      var args = arguments;

      return function(_) {
        return f.apply(this, Array.Concat(args, arguments));
      };
    };
  };

  // ApplyToObject calls a named method, or applies a named setter or getter,
  // depending on the type of the object member.
  this.ApplyToObject = function(object, member_name, value, _) {
    var item = object[member_name];
    if (typeof(item) == 'function')  // It's a function.
      return item.apply(object, arrayfromargs(arguments).slice(2));

    if (value != undefined)  // It's a setter.
      object[member_name] = item = value;

    return item;
  };

  // ObjectApplier() is a function that creates a function to call a method or
  // setter.
  this.ObjectApplier = this.Partial(this.ApplyToObject);
}();

#endif  // __FUNCTIONAL
